@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000004
+#define IP_VS_VERSION_CODE            0x000005
 #define NVERSION(version)                       \
         (version >> 16) & 0xFF,                 \
         (version >> 8) & 0xFF,                  \
@@ -37,8 +37,12 @@
 
 #define IP_VS_SCHEDNAME_MAXLEN         16
 
-
 struct ip_vs_rule_user {
+        /* global options */
+        int             tcp_timeout;    /* timeout values */
+        int             tcp_fin_timeout;
+        int             udp_timeout;
+        
 	/* virtual service options */
 	u_int16_t	protocol;
 	u_int32_t	vaddr;          /* virtual address */
@@ -66,7 +70,7 @@ struct ip_vs_rule_user {
  */
 #define MASQUERADE_EXPIRE_ICMP      125*HZ
 
-#define IP_VS_BASE_CTL  	128	    /* base for IPVS socket options */
+#define IP_VS_BASE_CTL  	(64+1024+64)    /* base for IPVS socket options */
 
 #define IP_VS_SO_SET_NONE	IP_VS_BASE_CTL	         /* just peek */
 #define IP_VS_SO_SET_INSERT	(IP_VS_BASE_CTL+1)
@@ -75,10 +79,11 @@ struct ip_vs_rule_user {
 #define IP_VS_SO_SET_DEL	(IP_VS_BASE_CTL+4)
 #define IP_VS_SO_SET_FLUSH	(IP_VS_BASE_CTL+5)
 #define IP_VS_SO_SET_LIST	(IP_VS_BASE_CTL+6)
-#define IP_VS_SO_SET_ADDDEST	(IP_VS_BASE_CTL+10)
-#define IP_VS_SO_SET_DELDEST	(IP_VS_BASE_CTL+11)
-#define IP_VS_SO_SET_EDITDEST	(IP_VS_BASE_CTL+12)
-#define IP_VS_SO_SET_MAX	IP_VS_SO_SET_EDITDEST
+#define IP_VS_SO_SET_ADDDEST	(IP_VS_BASE_CTL+7)
+#define IP_VS_SO_SET_DELDEST	(IP_VS_BASE_CTL+8)
+#define IP_VS_SO_SET_EDITDEST	(IP_VS_BASE_CTL+9)
+#define IP_VS_SO_SET_TIMEOUTS	(IP_VS_BASE_CTL+10)
+#define IP_VS_SO_SET_MAX	IP_VS_SO_SET_TIMEOUTS
 
 #define IP_VS_SO_GET_VERSION	IP_VS_BASE_CTL
 #define IP_VS_SO_GET_MAX	IP_VS_SO_GET_VERSION
@@ -347,9 +352,6 @@ struct ip_vs_app
  *      IPVS core functions
  *      (from ip_vs_core.c)
  */
-extern int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb);
-extern int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp);
-extern int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp);
 extern struct ip_vs_conn * ip_vs_schedule(struct ip_vs_service *svc,
                                           struct iphdr *iph);
 const char *vs_proto_name(unsigned proto);
@@ -374,6 +376,7 @@ const char *vs_proto_name(unsigned proto);
 #define VS_STATE_OUTPUT	        4
 #define VS_STATE_INPUT_ONLY	8
 
+extern struct ip_vs_timeout_table vs_timeout_table;
 extern struct ip_vs_timeout_table vs_timeout_table_dos;
 extern atomic_t ip_vs_conn_no_cport_cnt;
 
@@ -462,7 +465,6 @@ ip_vs_control_add(struct ip_vs_conn *cp, struct ip_vs_conn *ctl_cp)
 extern int register_ip_vs_app(struct ip_vs_app *mapp,
                               unsigned short proto, __u16 port);
 extern int unregister_ip_vs_app(struct ip_vs_app *mapp);
-extern struct ip_vs_app * ip_vs_app_get(unsigned short proto, __u16 port);
 extern struct ip_vs_app * ip_vs_bind_app(struct ip_vs_conn *cp);
 extern int ip_vs_unbind_app(struct ip_vs_conn *cp);
 extern int ip_vs_app_pkt_out(struct ip_vs_conn *, struct sk_buff **skb_p);
@@ -499,7 +501,7 @@ ip_vs_lookup_service(unsigned long fwmark,
                      __u16 protocol, __u32 vaddr, __u16 vport);
 extern struct ip_vs_service * ip_vs_lookup_svc_fwm(unsigned long fwmark);
 extern struct ip_vs_dest *
-__ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport);
+ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport);
 extern void update_defense_level(void);
 extern void ip_vs_random_dropentry(void);
 extern int ip_vs_control_init(void);
@@ -523,16 +525,6 @@ static __inline__ int ip_vs_todrop(void)
 }
 
 
-/*
- *	init function prototypes for scheduling modules
- *      these function will be called when they are built in kernel
- */
-extern int ip_vs_rr_init(void);
-extern int ip_vs_wrr_init(void);
-extern int ip_vs_lc_init(void);
-extern int ip_vs_wlc_init(void);
-
-
 /*
  *      Slow timer functions for IPVS
  *      (from ip_vs_timer.c)
