@@ -5,9 +5,9 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.5 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
  *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
  *
@@ -72,7 +72,7 @@ __ip_vs_set_expire(struct ip_vs_conn *cp, unsigned long tout)
 
 
 /*
- *  Fine locking granularity for big connection hash
+ *  Fine locking granularity for big connection hash table
  */
 #define CT_LOCKARRAY_BITS  4
 #define CT_LOCKARRAY_SIZE  (1<<CT_LOCKARRAY_BITS)
@@ -254,7 +254,7 @@ struct ip_vs_conn *ip_vs_conn_in_get
  *  Gets ip_vs_conn associated with supplied parameters in the ip_vs_table.
  *  Called for pkts coming from inside-to-OUTside.
  *	s_addr, s_port: pkt source address (inside host)
- *	d_addr, d_port: pkt dest address (foreigh host)
+ *	d_addr, d_port: pkt dest address (foreign host)
  *  Caller must lock tables
  */
 struct ip_vs_conn *ip_vs_conn_out_get
@@ -370,7 +370,7 @@ struct ip_vs_timeout_table vs_timeout_table_dos = {
  *	Under flood attack we switch to vs_timeout_table_dos
  */
 
-struct ip_vs_timeout_table *ip_vs_timeout_table = &vs_timeout_table;
+static struct ip_vs_timeout_table *ip_vs_timeout_table = &vs_timeout_table;
 
 
 static const char * state_name_table[IP_VS_S_LAST+1] = {
@@ -413,7 +413,7 @@ const char * ip_vs_state_name(int state)
 	return state_name_table[state] ? state_name_table[state] : "?";
 }
 
-struct vs_tcp_states_t vs_tcp_states [] = {
+static struct vs_tcp_states_t vs_tcp_states [] = {
 /*	INPUT */
 /*        sNO, sES, sSS, sSR, sFW, sTW, sCL, sCW, sLA, sLI, sSA	*/
 /*syn*/ {{sSR, sES, sES, sSR, sSR, sSR, sSR, sSR, sSR, sSR, sSR }},
@@ -436,7 +436,7 @@ struct vs_tcp_states_t vs_tcp_states [] = {
 /*rst*/ {{sCL, sCL, sCL, sSR, sCL, sCL, sCL, sCL, sLA, sLI, sCL }},
 };
 
-struct vs_tcp_states_t vs_tcp_states_dos [] = {
+static struct vs_tcp_states_t vs_tcp_states_dos [] = {
 /*	INPUT */
 /*        sNO, sES, sSS, sSR, sFW, sTW, sCL, sCW, sLA, sLI, sSA	*/
 /*syn*/ {{sSR, sES, sES, sSR, sSR, sSR, sSR, sSR, sSR, sSR, sSA }},
@@ -459,7 +459,7 @@ struct vs_tcp_states_t vs_tcp_states_dos [] = {
 /*rst*/ {{sCL, sCL, sCL, sSR, sCL, sCL, sCL, sCL, sLA, sLI, sCL }},
 };
 
-struct vs_tcp_states_t *ip_vs_state_table = vs_tcp_states;
+static struct vs_tcp_states_t *ip_vs_state_table = vs_tcp_states;
 
 void ip_vs_secure_tcp_set(int on)
 {
@@ -819,12 +819,16 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
         cp->control	   = NULL;
 	
 	atomic_set(&cp->n_control, 0);
-	atomic_set(&cp->refcnt, 0);
 	atomic_set(&cp->in_pkts, 0);
 
         /*
-         *  Hash it in the ip_vs_table
+         * Set the entry is referenced by the current thread before hashing
+         * it in the table, so that other thread run ip_vs_random_dropentry
+         * but cannot drop this entry.
          */
+	atomic_set(&cp->refcnt, 1);
+
+        /*  Hash it in the ip_vs_table */
         ip_vs_conn_hash(cp);
 
         if (flags & IP_VS_CONN_F_NO_CPORT)
@@ -833,9 +837,6 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
         ip_vs_bind_app(cp);
         vs_set_state_timeout(cp, IP_VS_S_NONE);
 
-        /*  Just increase its refcnt so that it can be put back */
-        atomic_inc(&cp->refcnt);
-
         return cp;
 }
 
@@ -939,12 +940,14 @@ void ip_vs_random_dropentry(void)
 	 * Randomly scan 1/32 of the whole table every second
 	 */
 	for (idx=0; idx<(IP_VS_CONN_TAB_SIZE>>5); idx++) {
+                unsigned hash = net_random()&IP_VS_CONN_TAB_MASK;
+
 		/*
 		 *  Lock is actually needed in this loop.
 		 */
-		ct_write_lock(idx);
+		ct_write_lock(hash);
 		
-		l = &ip_vs_table[net_random()&IP_VS_CONN_TAB_MASK];
+		l = &ip_vs_table[hash];
 		for (e=l->next; e!=l; e=e->next) {
 			cp = list_entry(e, struct ip_vs_conn, c_list);
 			if (cp->cport == 0)
@@ -968,7 +971,7 @@ void ip_vs_random_dropentry(void)
 			/*
 			 * Drop the entry, and drop its ct if not referenced
 			 */
-			ct_write_unlock(idx);
+			ct_write_unlock(hash);
 			IP_VS_DBG(1, "Drop connection\n");
 			ct = cp->control;
 			fn = (cp->timer).function;
@@ -979,9 +982,9 @@ void ip_vs_random_dropentry(void)
 				del_sltimer(&ct->timer);
 				fn((unsigned long)ct);
 			}
-			ct_write_lock(idx);
+			ct_write_lock(hash);
 		}
-		ct_write_unlock(idx);
+		ct_write_unlock(hash);
 	}
 }
 		
