@@ -5,9 +5,9 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.6 2000/07/30 04:10:05 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.7 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
  *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
  *
@@ -162,7 +162,7 @@ static struct sk_buff *vs_skb_cow(struct sk_buff **skb_p,
  *  or selects a server and creates a connection entry plus a template.
  *  Locking: we are svc user (svc->refcnt), so we hold all dests too
  */
-struct ip_vs_conn *
+static struct ip_vs_conn *
 ip_vs_sched_persist(struct ip_vs_service *svc, struct iphdr *iph)
 {
 	struct ip_vs_conn *cp = NULL;
@@ -429,10 +429,10 @@ struct ip_vs_conn *ip_vs_schedule(struct ip_vs_service *svc, struct iphdr *iph)
 
 /*
  *  Pass or drop the packet.
- *  Called by ip_vs_in, when the virtual service is avaiable but
+ *  Called by ip_vs_in, when the virtual service is available but
  *  no destination is available for a new connection.
  */
-int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
+static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 {
 	struct iphdr *iph = skb->nh.iph;
 	__u16 *portp = (__u16 *)&(((char *)iph)[iph->ihl*4]);
@@ -465,7 +465,7 @@ int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 /*
  *      NAT transmitter (only for outside-to-inside nat forwarding)
  */
-int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
+static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
         struct rtable *rt;			/* Route to the other host */
 	struct iphdr  *iph = skb->nh.iph;
@@ -477,7 +477,7 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	int csum_ok = 0;
         int mtu;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
         h.raw = (char*) iph + iph->ihl * 4;
 	size = ntohs(iph->tot_len) - (iph->ihl * 4);
@@ -602,12 +602,16 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
         IP_VS_DBG(1, "NAT to %u.%u.%u.%u:%d\n",
                   NIPQUAD(iph->daddr), ntohs(h.portp[1]));
 
-        /* Fixme: when application helper enlarges the packet and the length
+        /* FIXME: when application helper enlarges the packet and the length
            is larger than the MTU of outgoing device, there will be still
            MTU problem. */
+
+#ifdef CONFIG_NETFILTER_DEBUG
+        skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
+#endif /* CONFIG_NETFILTER_DEBUG */
         ip_send(skb);
 
-        LeaveFunction(1);
+        LeaveFunction(4);
 	return 0;
 
   tx_error_icmp:
@@ -633,7 +637,7 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
  *   Returns:
  *     if succeeded, return 0; otherwise, return -EINVAL.
  */
-int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
+static int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
 	struct rtable *rt;			/* Route to the other host */
 	struct net_device *tdev;		/* Device to other host */
@@ -646,7 +650,7 @@ int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	u32    src = 0;
 	int    mtu;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
         if (skb->protocol != __constant_htons(ETH_P_IP)) {
 		IP_VS_DBG(0, "ip_vs_tunnel_xmit(): protocol error, "
@@ -656,8 +660,8 @@ int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	}
 
 	if (ip_route_output(&rt, dst, src, RT_TOS(tos), 0)) {
-		IP_VS_DBG(0, "ip_vs_tunnel_xmit(): route error, dst: %08X\n",
-                          dst);
+		IP_VS_DBG(0, "ip_vs_tunnel_xmit(): route error, "
+                          "dest: %u.%u.%u.%u", NIPQUAD(dst));
 		goto tx_error_icmp;
 	}
 	tdev = rt->u.dst.dev;
@@ -709,10 +713,6 @@ int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	dst_release(skb->dst);
 	skb->dst = &rt->u.dst;
 
-        /* drop conntrack */
-/*          nf_conntrack_put(skb->nfct); */
-/*          skb->nfct = NULL; */
-
 	/*
 	 *	Push down and install the IPIP header.
 	 */
@@ -729,9 +729,12 @@ int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	ip_select_ident(iph, &rt->u.dst);
 	ip_send_check(iph);
 
+#ifdef CONFIG_NETFILTER_DEBUG
+        skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
+#endif /* CONFIG_NETFILTER_DEBUG */
 	ip_send(skb);
 
-        LeaveFunction(1);
+        LeaveFunction(4);
         
 	return 0;
 
@@ -754,18 +757,18 @@ static inline int do_ip_send(struct sk_buff *skb)
 /*
  *      Direct Routing
  */
-int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
+static int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 {
 	struct rtable *rt;			/* Route to the other host */
 	struct iphdr  *iph = skb->nh.iph;
 	u8     tos = iph->tos;
         int    mtu;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
         if (ip_route_output(&rt, cp->daddr, 0, RT_TOS(tos), 0)) {
-		IP_VS_DBG(0, "ip_vs_dr_xmit(): route error, dest: %08X\n",
-                          cp->daddr);
+		IP_VS_DBG(0, "ip_vs_dr_xmit(): route error, "
+                          "dest: %u.%u.%u.%u\n", NIPQUAD(cp->daddr));
 		goto tx_error_icmp;
 	}
 
@@ -782,18 +785,16 @@ int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
         dst_release(skb->dst);
 	skb->dst = &rt->u.dst;
 
-	/* drop conntrack */
-/*          nf_conntrack_put(skb->nfct); */
-/*          skb->nfct = NULL; */
-
+#ifdef CONFIG_NETFILTER_DEBUG
+        skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
+#endif /* CONFIG_NETFILTER_DEBUG */
         ip_send(skb);
 
 #if 0000
-        skb->nf_debug = 0;
         NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, skb, NULL, rt->u.dst.dev,
                 do_ip_send);
 #endif
-        LeaveFunction(1);
+        LeaveFunction(4);
 	return 0;
 
   tx_error_icmp:
@@ -854,7 +855,7 @@ static inline int ip_vs_forward(struct sk_buff *skb, struct ip_vs_conn *cp)
  *	Currently handles error types - unreachable, quench, ttl exceeded.
  *      (Only used in VS/NAT)
  */
-int ip_vs_out_icmp(struct sk_buff **skb_p)
+static int ip_vs_out_icmp(struct sk_buff **skb_p)
 {
 	struct sk_buff	*skb   = *skb_p;
 	struct iphdr	*iph   = skb->nh.iph;
@@ -972,7 +973,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 	int csum = 0;
 	int csum_ok = 0;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
 	h.raw = (char*) iph + iph->ihl * 4;
 	size = ntohs(iph->tot_len) - (iph->ihl * 4);
@@ -1002,6 +1003,19 @@ static unsigned int ip_vs_out(unsigned int hooknum,
         cp = ip_vs_conn_out_get(iph->protocol, iph->saddr, h.portp[0],
                                 iph->daddr, h.portp[1]);
         if (!cp) {
+		if (ip_vs_lookup_real_service(iph->protocol,
+                                              iph->saddr, h.portp[0])) {
+			/*
+			 * Notify the real server: there is no existing
+                         * entry if it is not RST packet or not TCP packet.
+			 */
+			if (!h.th->rst || iph->protocol != IPPROTO_TCP) {
+				icmp_send(skb, ICMP_DEST_UNREACH,
+                                          ICMP_PORT_UNREACH, 0);
+				kfree_skb(skb);
+				return NF_STOLEN;
+			}
+		}
                 IP_VS_DBG(2, "packet for %s %d.%d.%d.%d:%d "
                           "continue traversal as normal.\n",
                           vs_proto_name(iph->protocol),
@@ -1117,7 +1131,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 	vs_set_state(cp, VS_STATE_OUTPUT, iph, h.portp);
 	ip_vs_conn_put(cp);
         
-        LeaveFunction(1);
+        LeaveFunction(4);
 	return NF_ACCEPT;
 }
 
@@ -1128,7 +1142,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
  *	forward to the right destination host if relevant.
  *	Currently handles error types - unreachable, quench, ttl exceeded
  */
-int ip_vs_in_icmp(struct sk_buff **skb_p)
+static int ip_vs_in_icmp(struct sk_buff **skb_p)
 {
         struct sk_buff 	*skb   = *skb_p;
  	struct iphdr	*iph   = skb->nh.iph;
@@ -1247,10 +1261,9 @@ int ip_vs_in_icmp(struct sk_buff **skb_p)
         dst_release(skb->dst);
         skb->dst = &rt->u.dst;
 
-/*          skb->nfcache |= NFC_ALTERED; */
-/*          nf_conntrack_put(skb->nfct); */
-/*          skb->nfct = NULL; */
-
+#ifdef CONFIG_NETFILTER_DEBUG
+        skb->nf_debug = 1 << NF_IP_LOCAL_OUT;
+#endif /* CONFIG_NETFILTER_DEBUG */
         ip_send(skb);
         return NF_STOLEN;
 
@@ -1354,9 +1367,8 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 		}
                 
 		/*
-		 * Let the virtual server select a real server
-		 * for the incomming connection, and create a
-		 * connection entry.
+		 * Let the virtual server select a real server for the
+                 * incomming connection, and create a connection entry.
 		 */
 		cp = ip_vs_schedule(svc, iph);
 		if (!cp)
@@ -1384,11 +1396,11 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 }
 
 
-/* After packet filtering, forward packet through VS/DR, VS/TUN,
+/* Before packet filtering, forward packet through VS/DR, VS/TUN,
    or VS/NAT(change destination) */
 static struct nf_hook_ops ip_vs_in_ops = {
         { NULL, NULL },
-        ip_vs_in, PF_INET, /*  NF_IP_PRE_ROUTING */NF_IP_LOCAL_IN, 100
+        ip_vs_in, PF_INET, /*  NF_IP_PRE_ROUTING */NF_IP_LOCAL_IN, -100
 };
 
 /* After packet filtering, change source only for VS/NAT */
@@ -1401,15 +1413,11 @@ static struct nf_hook_ops ip_vs_out_ops = {
 /*
  *	Initialize IP Virtual Server
  */
-#ifdef MODULE
-int ip_vs_init(void)
-#else
-int __init ip_vs_init(void)
-#endif
+static int __init ip_vs_init(void)
 {
 	int ret;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
         ret = ip_vs_control_init();
 	if (ret < 0) {
@@ -1442,20 +1450,9 @@ int __init ip_vs_init(void)
 		goto cleanup_inops;
 	}
 
-#ifdef CONFIG_IP_VS_RR
-	ip_vs_rr_init();
-#endif
-#ifdef CONFIG_IP_VS_WRR
-	ip_vs_wrr_init();
-#endif
-#ifdef CONFIG_IP_VS_LC
-	ip_vs_lc_init();
-#endif
-#ifdef CONFIG_IP_VS_WLC
-	ip_vs_wlc_init();
-#endif
+        IP_VS_INFO("ipvs loaded.\n");
 
-        LeaveFunction(1);
+        LeaveFunction(4);
         
 	return ret;
 
@@ -1472,8 +1469,7 @@ int __init ip_vs_init(void)
 	return ret;
 }
 
-
-int ip_vs_cleanup(void)
+static void __exit ip_vs_cleanup(void)
 {
 	nf_unregister_hook(&ip_vs_out_ops);
 	nf_unregister_hook(&ip_vs_in_ops);
@@ -1481,29 +1477,8 @@ int ip_vs_cleanup(void)
 	ip_vs_conn_cleanup();
 	ip_vs_sltimer_cleanup();
 	ip_vs_control_cleanup();
-	return 0;
+        IP_VS_INFO("ipvs unloaded.\n");
 }
 
-
-#ifdef MODULE
-EXPORT_NO_SYMBOLS;
-
-int init_module(void)
-{
-	if(ip_vs_init() != 0)
-		return -EIO;
-
-	IP_VS_INFO("ipvs module loaded.\n");
-
-	return 0;
-}
-
-void cleanup_module(void)
-{
-	if(ip_vs_cleanup() != 0)
-		IP_VS_INFO("cannot remove ipvs module\n");
-	else
-		IP_VS_INFO("ipvs module unloaded.\n");
-}
-
-#endif /* MODULE */
+module_init(ip_vs_init);
+module_exit(ip_vs_cleanup);
