@@ -5,9 +5,9 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.5 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
  *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
  *
@@ -23,6 +23,7 @@
 #include <linux/config.h>
 #include <linux/kernel.h>
 #include <linux/module.h>
+#include <linux/init.h>
 #include <linux/types.h>
 #include <linux/errno.h>
 #include <linux/fs.h>
@@ -44,40 +45,42 @@
 //#include <net/ip_vs.h>
 
 /* spinlock for do_ip_vs_set_ctl */
-spinlock_t __ip_vs_mutex = SPIN_LOCK_UNLOCKED;
+static spinlock_t __ip_vs_mutex = SPIN_LOCK_UNLOCKED;
 
 /* lock for service table */
-rwlock_t __ip_vs_svc_lock = RW_LOCK_UNLOCKED;
+static rwlock_t __ip_vs_svc_lock = RW_LOCK_UNLOCKED;
 
 /* lock for table with the real services */
-rwlock_t __ip_vs_rs_lock = RW_LOCK_UNLOCKED;
+static rwlock_t __ip_vs_rs_lock = RW_LOCK_UNLOCKED;
 
 /* lock for state and timeout tables */
-rwlock_t __ip_vs_securetcp_lock = RW_LOCK_UNLOCKED;
+static rwlock_t __ip_vs_securetcp_lock = RW_LOCK_UNLOCKED;
 
 /* lock for drop entry handling */
-spinlock_t __ip_vs_dropentry_lock = SPIN_LOCK_UNLOCKED;
+static spinlock_t __ip_vs_dropentry_lock = SPIN_LOCK_UNLOCKED;
 
 /* lock for drop packet handling */
-spinlock_t __ip_vs_droppacket_lock = SPIN_LOCK_UNLOCKED;
+static spinlock_t __ip_vs_droppacket_lock = SPIN_LOCK_UNLOCKED;
 
-int ip_vs_drop_rate = 0;
-int ip_vs_drop_counter = 0;
-atomic_t ip_vs_dropentry = ATOMIC_INIT(0);
+static int ip_vs_drop_rate = 0;
+static int ip_vs_drop_counter = 0;
+static atomic_t ip_vs_dropentry = ATOMIC_INIT(0);
 
 /* sysctl variables */
-int sysctl_ip_vs_debug_level = 0;
-int sysctl_ip_vs_drop_entry = 0;
-int sysctl_ip_vs_drop_packet = 0;
-int sysctl_ip_vs_secure_tcp = 0;
-int sysctl_ip_vs_amemthresh = 1024;
-int sysctl_ip_vs_am_droprate = 10;
+static int sysctl_ip_vs_drop_entry = 0;
+static int sysctl_ip_vs_drop_packet = 0;
+static int sysctl_ip_vs_secure_tcp = 0;
+static int sysctl_ip_vs_amemthresh = 1024;
+static int sysctl_ip_vs_am_droprate = 10;
+
+#ifdef CONFIG_IP_VS_DEBUG
+static int sysctl_ip_vs_debug_level = 0;
 
 int ip_vs_get_debug_level(void)
 {
         return sysctl_ip_vs_debug_level;
 }
-
+#endif
 
 /*
  *  update_defense_level is called from timer bh and from sysctl.
@@ -186,9 +189,9 @@ void update_defense_level(void)
 #define IP_VS_SVC_TAB_MASK (IP_VS_SVC_TAB_SIZE - 1)
 
 /* the service table hashed by <protocol, addr, port> */
-struct list_head ip_vs_svc_table[IP_VS_SVC_TAB_SIZE];
+static struct list_head ip_vs_svc_table[IP_VS_SVC_TAB_SIZE];
 /* the service table hashed by fwmark */
-struct list_head ip_vs_svc_fwm_table[IP_VS_SVC_TAB_SIZE];
+static struct list_head ip_vs_svc_fwm_table[IP_VS_SVC_TAB_SIZE];
 
 /*
  *  Hash table: for real service lookups
@@ -197,18 +200,18 @@ struct list_head ip_vs_svc_fwm_table[IP_VS_SVC_TAB_SIZE];
 #define IP_VS_RTAB_SIZE (1 << IP_VS_RTAB_BITS)
 #define IP_VS_RTAB_MASK (IP_VS_RTAB_SIZE - 1)
 
-struct list_head ip_vs_rtable[IP_VS_RTAB_SIZE];
+static struct list_head ip_vs_rtable[IP_VS_RTAB_SIZE];
 
 /*
  * Trash for destinations
  */
-LIST_HEAD(ip_vs_dest_trash);
+static LIST_HEAD(ip_vs_dest_trash);
 
 /*
  * FTP & NULL virtual service counters
  */
-atomic_t ip_vs_ftpsvc_counter = ATOMIC_INIT(0);
-atomic_t ip_vs_nullsvc_counter = ATOMIC_INIT(0);
+static atomic_t ip_vs_ftpsvc_counter = ATOMIC_INIT(0);
+static atomic_t ip_vs_nullsvc_counter = ATOMIC_INIT(0);
 
 
 /*
@@ -237,7 +240,7 @@ static __inline__ unsigned ip_vs_svc_fwm_hashkey(__u32 fwmark)
  *  Should be called with locked tables.
  *  Returns bool success.
  */
-int ip_vs_svc_hash(struct ip_vs_service *svc)
+static int ip_vs_svc_hash(struct ip_vs_service *svc)
 {
 	unsigned hash;
 
@@ -271,7 +274,7 @@ int ip_vs_svc_hash(struct ip_vs_service *svc)
  *  Should be called with locked tables.
  *  Returns bool success.
  */
-int ip_vs_svc_unhash(struct ip_vs_service *svc)
+static int ip_vs_svc_unhash(struct ip_vs_service *svc)
 {
 	if (!(svc->flags & IP_VS_SVC_F_HASHED)) {
 		IP_VS_ERR("ip_vs_svc_unhash(): request for unhash flagged, "
@@ -408,12 +411,11 @@ ip_vs_lookup_service(unsigned long fwmark, __u16 protocol, __u32 vaddr, __u16 vp
 /*
  *  Returns hash value for real service
  */
-static __inline__ unsigned
-ip_vs_rs_hashkey(unsigned proto, __u32 addr, __u16 port)
+static __inline__ unsigned ip_vs_rs_hashkey(__u32 addr, __u16 port)
 {
 	register unsigned porth = ntohs(port);
 	
-	return (proto^ntohl(addr)^(porth>>IP_VS_RTAB_BITS)^porth)
+	return (ntohl(addr)^(porth>>IP_VS_RTAB_BITS)^porth)
 		& IP_VS_RTAB_MASK;
 }
 
@@ -422,7 +424,7 @@ ip_vs_rs_hashkey(unsigned proto, __u32 addr, __u16 port)
  *  should be called with locked tables.
  *  returns bool success.
  */
-int ip_vs_rs_hash(struct ip_vs_dest *dest)
+static int ip_vs_rs_hash(struct ip_vs_dest *dest)
 {
 	unsigned hash;
 
@@ -434,7 +436,7 @@ int ip_vs_rs_hash(struct ip_vs_dest *dest)
 	 *	Hash by proto,addr,port,
 	 *	which are the parameters of the real service.
 	 */
-	hash = ip_vs_rs_hashkey(0, dest->addr, dest->port);
+	hash = ip_vs_rs_hashkey(dest->addr, dest->port);
 	list_add(&dest->d_list, &ip_vs_rtable[hash]);
 
 	return 1;
@@ -445,7 +447,7 @@ int ip_vs_rs_hash(struct ip_vs_dest *dest)
  *  should be called with locked tables.
  *  returns bool success.
  */
-int ip_vs_rs_unhash(struct ip_vs_dest *dest)
+static int ip_vs_rs_unhash(struct ip_vs_dest *dest)
 {
 	/*
 	 * Remove it from the ip_vs_rtable table.
@@ -472,7 +474,7 @@ ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport)
 	 *	Check for "full" addressed entries
 	 *	Return the first found entry
 	 */
-	hash = ip_vs_rs_hashkey(0, daddr, dport);
+	hash = ip_vs_rs_hashkey(daddr, dport);
 	
 	l = &ip_vs_rtable[hash];
 	
@@ -496,7 +498,7 @@ ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport)
 /*
  *  Lookup destination by {addr,port} in the given service
  */
-struct ip_vs_dest *
+static struct ip_vs_dest *
 ip_vs_lookup_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 {
 	struct ip_vs_dest *dest;
@@ -528,7 +530,7 @@ ip_vs_lookup_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
  *  continue, and the counting information of the dest is also useful for
  *  scheduling.
  */
-struct ip_vs_dest *
+static struct ip_vs_dest *
 ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 {
 	struct ip_vs_dest *dest;
@@ -579,9 +581,9 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 /*
  *  Update a destination in the given service
  */
-void __ip_vs_update_dest(struct ip_vs_service *svc,
-                         struct ip_vs_dest *dest,
-			 struct ip_vs_rule_user *ur)
+static void __ip_vs_update_dest(struct ip_vs_service *svc,
+				struct ip_vs_dest *dest,
+				struct ip_vs_rule_user *ur)
 {
 	int conn_flags;
 
@@ -628,12 +630,12 @@ void __ip_vs_update_dest(struct ip_vs_service *svc,
 /*
  *  Create a destination for the given service
  */
-struct ip_vs_dest *
+static struct ip_vs_dest *
 ip_vs_new_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 {
 	struct ip_vs_dest *dest;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
 	dest = (struct ip_vs_dest*) kmalloc(sizeof(struct ip_vs_dest),
 					GFP_ATOMIC);
@@ -657,7 +659,7 @@ ip_vs_new_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 	INIT_LIST_HEAD(&dest->d_list);
 	__ip_vs_update_dest(svc, dest, ur);
 
-	LeaveFunction(1);
+	LeaveFunction(4);
 
 	return dest;
 }
@@ -666,13 +668,14 @@ ip_vs_new_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 /*
  *  Add a destination into an existing service
  */
-int ip_vs_add_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
+static int ip_vs_add_dest(struct ip_vs_service *svc,
+			  struct ip_vs_rule_user *ur)
 {
 	struct ip_vs_dest *dest;
 	__u32 daddr = ur->daddr;
 	__u16 dport = ur->dport;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
 	if (ur->weight < 0) {
 		IP_VS_ERR("ip_vs_add_dest(): server weight less than zero\n");
@@ -746,7 +749,7 @@ int ip_vs_add_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 
 	write_unlock_bh(&__ip_vs_svc_lock);
 
-        LeaveFunction(1);
+        LeaveFunction(4);
         
 	return 0;
 }
@@ -755,13 +758,14 @@ int ip_vs_add_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 /*
  *  Edit a destination in the given service
  */
-int ip_vs_edit_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
+static int ip_vs_edit_dest(struct ip_vs_service *svc,
+			   struct ip_vs_rule_user *ur)
 {
 	struct ip_vs_dest *dest;
 	__u32 daddr = ur->daddr;
 	__u16 dport = ur->dport;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
 	if (ur->weight < 0) {
 		IP_VS_ERR("ip_vs_add_dest(): server weight less than zero\n");
@@ -779,7 +783,7 @@ int ip_vs_edit_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 
 	__ip_vs_update_dest(svc, dest, ur);
 
-        LeaveFunction(1);
+        LeaveFunction(4);
         
 	return 0;
 }
@@ -789,7 +793,7 @@ int ip_vs_edit_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
  *  Delete a destination (must be already unlinked from the service)
  */
 
-void __ip_vs_del_dest(struct ip_vs_dest *dest)
+static void __ip_vs_del_dest(struct ip_vs_dest *dest)
 {
 	/*
 	 *  Remove it from the d-linked list with the real services.
@@ -818,8 +822,9 @@ void __ip_vs_del_dest(struct ip_vs_dest *dest)
 /*
  *  Unlink a destination from the given service
  */
-void __ip_vs_unlink_dest(struct ip_vs_service *svc,struct ip_vs_dest *dest,
-			int svcupd)
+static void __ip_vs_unlink_dest(struct ip_vs_service *svc,
+				struct ip_vs_dest *dest,
+				int svcupd)
 {
 	dest->flags &= ~IP_VS_DEST_F_AVAILABLE;
 
@@ -836,13 +841,13 @@ void __ip_vs_unlink_dest(struct ip_vs_service *svc,struct ip_vs_dest *dest,
 }
 
 
-int ip_vs_del_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
+static int ip_vs_del_dest(struct ip_vs_service *svc,struct ip_vs_rule_user *ur)
 {
 	struct ip_vs_dest *dest;
 	__u32 daddr = ur->daddr;
 	__u16 dport = ur->dport;
 
-	EnterFunction(1);
+	EnterFunction(4);
 
 	/*
 	 *    Lookup the destination list
@@ -871,7 +876,7 @@ int ip_vs_del_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 	 */
 	__ip_vs_del_dest(dest);
 
-        LeaveFunction(1);
+        LeaveFunction(4);
 
 	return 0;
 }
@@ -880,7 +885,7 @@ int ip_vs_del_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 /*
  *  Add a service into the service hash table
  */
-int ip_vs_add_service(struct ip_vs_rule_user *ur)
+static int ip_vs_add_service(struct ip_vs_rule_user *ur)
 {
 	__u16  protocol = ur->protocol;
 	__u32 vaddr = ur->vaddr;
@@ -968,7 +973,8 @@ int ip_vs_add_service(struct ip_vs_rule_user *ur)
 /*
  *	Edit a service and bind it with a new scheduler
  */
-int ip_vs_edit_service(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
+static int ip_vs_edit_service(struct ip_vs_service *svc,
+			      struct ip_vs_rule_user *ur)
 {
 	struct ip_vs_scheduler *sched, *old_sched;
 
@@ -1027,7 +1033,7 @@ int ip_vs_edit_service(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
  *  Delete a service from the service list
  *  The service must be unlinked, unlocked and not referenced!
  */
-int __ip_vs_del_service(struct ip_vs_service *svc)
+static int __ip_vs_del_service(struct ip_vs_service *svc)
 {
 	struct list_head *l;
 	struct ip_vs_dest *dest;
@@ -1070,7 +1076,7 @@ int __ip_vs_del_service(struct ip_vs_service *svc)
 /*
  *  Delete a service from the service list
  */
-int ip_vs_del_service(struct ip_vs_service *svc)
+static int ip_vs_del_service(struct ip_vs_service *svc)
 {
 	if (svc == NULL)
 		return -EEXIST;
@@ -1095,7 +1101,7 @@ int ip_vs_del_service(struct ip_vs_service *svc)
 /*
  *  Flush all the virtual services
  */
-int ip_vs_flush(void)
+static int ip_vs_flush(void)
 {
 	int idx;
 	struct ip_vs_service *svc;
@@ -1177,12 +1183,14 @@ struct ip_vs_sysctl_table {
 
 static struct ip_vs_sysctl_table ipv4_vs_table = {
         NULL,
-	{{NET_IPV4_VS_DEBUG_LEVEL, "debug_level",
-          &sysctl_ip_vs_debug_level, sizeof(int), 0644, NULL,
-          &proc_dointvec},
-         {NET_IPV4_VS_AMEMTHRESH, "amemthresh",
+	{{NET_IPV4_VS_AMEMTHRESH, "amemthresh",
           &sysctl_ip_vs_amemthresh, sizeof(int), 0644, NULL,
           &proc_dointvec},
+#ifdef CONFIG_IP_VS_DEBUG
+         {NET_IPV4_VS_DEBUG_LEVEL, "debug_level",
+          &sysctl_ip_vs_debug_level, sizeof(int), 0644, NULL,
+          &proc_dointvec},
+#endif
          {NET_IPV4_VS_AMDROPRATE, "am_droprate",
           &sysctl_ip_vs_am_droprate, sizeof(int), 0644, NULL,
           &proc_dointvec},
@@ -1397,6 +1405,34 @@ static int ip_vs_get_info(char *buf, char **start, off_t offset, int length)
 }
 
 
+/*
+ * Set timeout values for tcp tcpfin udp in the vs_timeout_table.
+ */
+static int ip_vs_set_timeouts(struct ip_vs_rule_user *u)
+{
+        IP_VS_DBG(1, "Setting timeout tcp:%d tcpfin:%d udp:%d\n",
+                  u->tcp_timeout/HZ,
+                  u->tcp_fin_timeout/HZ,
+                  u->udp_timeout/HZ);
+
+        if (u->tcp_timeout) {
+                vs_timeout_table.timeout[IP_VS_S_ESTABLISHED]
+                        = u->tcp_timeout;
+        }
+
+        if (u->tcp_fin_timeout) {
+                vs_timeout_table.timeout[IP_VS_S_FIN_WAIT]
+                        = u->tcp_fin_timeout;
+        }
+
+        if (u->udp_timeout) {
+                vs_timeout_table.timeout[IP_VS_S_UDP]
+                        = u->udp_timeout;
+        }
+        return 0;
+}
+
+                                
 static int
 do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 {
@@ -1427,17 +1463,18 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 
 	spin_lock(&__ip_vs_mutex);
 
-	/*
-	 * Flush all the virtual service...
-	 */
-	if (cmd == IP_VS_SO_SET_FLUSH) {
+        if (cmd == IP_VS_SO_SET_FLUSH) {
+                /* Flush the virtual service */
 		ret = ip_vs_flush();
 		goto out_unlock;
-	}
+        } else if (cmd == IP_VS_SO_SET_TIMEOUTS) {
+                /* Set timeout values for (tcp tcpfin udp) */
+                ret = ip_vs_set_timeouts(urule);
+                goto out_unlock;
+        }
 	
 	/*
-	 * Check for valid protocol: TCP or UDP. Even
-	 * for fwmark!=0
+	 * Check for valid protocol: TCP or UDP. Even for fwmark!=0
 	 */
 	if (urule->protocol!=IPPROTO_TCP && urule->protocol!=IPPROTO_UDP) {
 		IP_VS_INFO("vs_ctl: invalid protocol: %d %d.%d.%d.%d:%d %s",
@@ -1448,7 +1485,7 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 	}
 
 	/*
-	 * Lookup the exact service by (protocol, vaddr, vport)
+	 * Lookup the exact service by (protocol, vaddr, vport) or fwmark
 	 */
 	if (urule->vfwmark == 0)
 		svc = __ip_vs_lookup_service(urule->protocol,
@@ -1538,12 +1575,12 @@ static struct nf_sockopt_ops ip_vs_sockopts = {
 };
 
 
-int ip_vs_control_init(void)
+int __init ip_vs_control_init(void)
 {
         int ret;
         int idx;
 
-        EnterFunction(1);
+        EnterFunction(4);
         
  	ret = nf_register_sockopt(&ip_vs_sockopts);
 	if (ret) {
@@ -1567,15 +1604,15 @@ int ip_vs_control_init(void)
                 INIT_LIST_HEAD(&ip_vs_rtable[idx]);
         }
 
-        LeaveFunction(1);
+        LeaveFunction(4);
         return 0;
 }
 
-void ip_vs_control_cleanup(void)
+void __exit ip_vs_control_cleanup(void)
 {
-        EnterFunction(1);
+        EnterFunction(4);
         unregister_sysctl_table(ipv4_vs_table.sysctl_header);
         proc_net_remove("ip_vs");
         nf_unregister_sockopt(&ip_vs_sockopts);
-        LeaveFunction(1);
+        LeaveFunction(4);
 }
