@@ -1,9 +1,9 @@
 /*
  * IPVS:        Round-Robin Scheduling module
  *
- * Version:     $Id: ip_vs_rr.c,v 1.2 2000/05/31 02:36:21 wensong Exp $
+ * Version:     $Id: ip_vs_rr.c,v 1.3 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
  *
  *              This program is free software; you can redistribute it and/or
@@ -23,9 +23,7 @@
 
 #include <linux/config.h>
 #include <linux/module.h>
-#ifdef CONFIG_KMOD
-#include <linux/kmod.h>
-#endif
+#include <linux/init.h>
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
@@ -106,30 +104,16 @@ static struct ip_vs_scheduler ip_vs_rr_scheduler = {
 	ip_vs_rr_schedule,	/* select a server from the destination list */
 };
 
-
-#ifdef MODULE
-int ip_vs_rr_init(void)
-#else
-int __init ip_vs_rr_init(void)
-#endif
+static int __init ip_vs_rr_init(void)
 {
-        IP_VS_INFO("Initializing RR scheduling\n");
         INIT_LIST_HEAD(&ip_vs_rr_scheduler.n_list);
-        return register_ip_vs_scheduler(&ip_vs_rr_scheduler) ;
+        return register_ip_vs_scheduler(&ip_vs_rr_scheduler);
 }
 
-
-#ifdef MODULE
-EXPORT_NO_SYMBOLS;
-
-int init_module(void)
+static void __exit ip_vs_rr_cleanup(void)
 {
-	return ip_vs_register_scheduler_module(&ip_vs_rr_scheduler);
+	unregister_ip_vs_scheduler(&ip_vs_rr_scheduler);
 }
 
-void cleanup_module(void)
-{
-	ip_vs_unregister_scheduler_module(&ip_vs_rr_scheduler);
-}
-
-#endif /* MODULE */
+module_init(ip_vs_rr_init);
+module_exit(ip_vs_rr_cleanup);
