@@ -1,9 +1,9 @@
 /*
  * IPVS:        Weighted Round-Robin Scheduling module
  *
- * Version:     $Id: ip_vs_wrr.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
+ * Version:     $Id: ip_vs_wrr.c,v 1.5 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
  *              This program is free software; you can redistribute it and/or
  *              modify it under the terms of the GNU General Public License
@@ -15,14 +15,14 @@
  *     Wensong Zhang            :     changed some comestics things for debugging
  *     Wensong Zhang            :     changed for the d-linked destination list
  *     Wensong Zhang            :     added the ip_vs_wrr_update_svc
+ *     Julian Anastasov         :     fixed the bug of returning destination
+ *                                    with weight 0 when all weights are zero
  *
  */
 
 #include <linux/config.h>
 #include <linux/module.h>
-#ifdef CONFIG_KMOD
-#include <linux/kmod.h>
-#endif
+#include <linux/init.h>
 #include <linux/types.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
@@ -79,7 +79,7 @@ static int ip_vs_wrr_update_svc(struct ip_vs_service *svc)
 /*
  *    Get the maximum weight of the service destinations.
  */
-int ip_vs_wrr_max_weight(struct ip_vs_service *svc)
+static int ip_vs_wrr_max_weight(struct ip_vs_service *svc)
 {
 	register struct list_head *l, *e;
 	struct ip_vs_dest *dest;
@@ -129,7 +129,8 @@ static struct ip_vs_dest* ip_vs_wrr_schedule(struct ip_vs_service *svc)
 				 * Still zero, which means no availabe servers.
 				 */
 				if (mark->cw == 0) {
-					write_unlock(&svc->sched_lock);
+					mark->cl = &svc->destinations;
+                                        write_unlock(&svc->sched_lock);
 					IP_VS_INFO("ip_vs_wrr_schedule(): "
 						   "no available servers\n");
 					return NULL;
@@ -170,29 +171,16 @@ static struct ip_vs_scheduler ip_vs_wrr_scheduler = {
 	ip_vs_wrr_schedule,	/* select a server from the destination list */
 };
 
-
-#ifdef MODULE
-int ip_vs_wrr_init(void)
-#else
-int __init ip_vs_wrr_init(void)
-#endif
+static int __init ip_vs_wrr_init(void)
 {
-	IP_VS_INFO("Initializing WRR scheduling\n");
 	INIT_LIST_HEAD(&ip_vs_wrr_scheduler.n_list);
 	return register_ip_vs_scheduler(&ip_vs_wrr_scheduler) ;
 }
 
-#ifdef MODULE
-EXPORT_NO_SYMBOLS;
-
-int init_module(void)
+static void __exit ip_vs_wrr_cleanup(void)
 {
-	return ip_vs_register_scheduler_module(&ip_vs_wrr_scheduler);
+	unregister_ip_vs_scheduler(&ip_vs_wrr_scheduler);
 }
 
-void cleanup_module(void)
-{
-	ip_vs_unregister_scheduler_module(&ip_vs_wrr_scheduler);
-}
-
-#endif /* MODULE */
+module_init(ip_vs_wrr_init);
+module_exit(ip_vs_wrr_cleanup);
