@@ -1,7 +1,7 @@
 /*
  *      ipvsadm - IP Virtual Server ADMinistration program
  *
- *      Version: $Id: ipvsadm.c,v 1.2 2000/07/06 14:57:09 wensong Exp $
+ *      Version: $Id: ipvsadm.c,v 1.3 2000/09/06 08:36:36 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@iinchina.net>
  *               Peter Kese <peter.kese@ijs.si>
@@ -42,6 +42,8 @@
  *        Horms               :   Specifying command and option options
  *                                (e.g. -Ln or -At) in one short option
  *                                with popt problem fixed.
+ *        Wensong Zhang       :   split the process_options and make
+ *                                two versions of parse_options.
  *
  *
  *      ippfvsadm - Port Fowarding & Virtual Server ADMinistration program
@@ -109,13 +111,13 @@
 #define IPVS_OPTION_PROCESSING          "getopt_long"
 #endif
 
-#define IPVSADM_VERSION_NO              "v1.10"
-#define IPVSADM_VERSION_DATE            "2000/05/25"
+#define IPVSADM_VERSION_NO              "v1.12"
+#define IPVSADM_VERSION_DATE            "2000/09/06"
 #define IPVSADM_VERSION         IPVSADM_VERSION_NO " " IPVSADM_VERSION_DATE
 
 #define MINIMUM_IPVS_VERSION_MAJOR      0
 #define MINIMUM_IPVS_VERSION_MINOR      0
-#define MINIMUM_IPVS_VERSION_PATCH      1
+#define MINIMUM_IPVS_VERSION_PATCH      5
 
 #ifndef IPVS_VERSION
 #define IPVS_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
@@ -222,6 +224,8 @@ int parse_options(int argc, char **argv, int reading_stdin,
         {"edit-server", 'e', POPT_ARG_NONE, NULL, 'e'};
         struct poptOption delete_server_option =
         {"delete-server", 'd', POPT_ARG_NONE, NULL, 'd'};
+        struct poptOption set_option =
+        {"set", 's', POPT_ARG_NONE, NULL, 's'};
         struct poptOption help_option =
         {"help", 'h', POPT_ARG_NONE, NULL, 'h'};
         struct poptOption read_stdin_option =
@@ -272,6 +276,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
 		add_server_option,
 		edit_server_option,
 		delete_server_option,
+                set_option,
 		help_option,
 		read_stdin_option,
 		write_stdout_option,
@@ -374,6 +379,10 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 *op = IP_VS_SO_SET_LIST;
 		options_sub=options_list;
                 break;
+        case 's':
+                *op = IP_VS_SO_SET_TIMEOUTS;
+		options_sub=options_NULL;
+		break;
 	case 'R':
 		read_stdin=1;
 		options_sub=options_NULL;
@@ -560,6 +569,25 @@ int parse_options(int argc, char **argv, int reading_stdin,
        		if (parse == 0)
                         fail(2, "illegal timeout for persistent service");
 	}
+
+        if (*op == IP_VS_SO_SET_TIMEOUTS) {
+                char *optarg1, *optarg2;
+                        
+                if ((optarg=(char *)poptGetArg(context))
+                    && (optarg1=(char *)poptGetArg(context))
+                    && (optarg2=(char *)poptGetArg(context))) {
+                        parse = parse_timeout(optarg, &ur->tcp_timeout)
+                                && parse_timeout(optarg1, &ur->tcp_fin_timeout)
+                                && parse_timeout(optarg2, &ur->udp_timeout);
+                        if (parse == 0)
+                                fail(2, "illegal timeout values");
+                } else
+                        fail(2, "-s option requires 3 timeout values");
+        }
+        
+        if (poptGetArg(context))
+                fail(2, "unexpected arguments");
+    
         poptFreeContext(context);
         return 0;
 }
@@ -586,6 +614,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
         	{"edit-server", 0, 0, 'e'},
         	{"delete-server", 0, 0, 'd'},
         	{"help", 0, 0, 'h'},
+        	{"set", 1, 0, 's'},
         	{"tcp-service", 1, 0, 't'},
         	{"udp-service", 1, 0, 'u'},
         	{"fwmark-service", 1, 0, 'f'},
@@ -611,7 +640,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
 	/* Re-process the arguments each time options is called*/
 	optind = 1;
 
-	if ((cmd = getopt_long(argc, argv, "AEDCaedlLh",
+	if ((cmd = getopt_long(argc, argv, "AEDCaedlLhs:",
                                long_options, NULL)) == EOF)
 		usage_exit(argv[0], -1);
 
@@ -649,9 +678,26 @@ int parse_options(int argc, char **argv, int reading_stdin,
                 *op = IP_VS_SO_SET_LIST;
                 optstr = "cn";
                 break;
+        case 's':
+                *op = IP_VS_SO_SET_TIMEOUTS;
+                optstr = "";
+                if (optind + 1 < argc
+                    && argv[optind][0] != '!'
+                    && argv[optind][0] != '-'
+                    && argv[optind+1][0] != '!'
+                    && argv[optind+1][0] != '-') {
+                        parse = parse_timeout(optarg, &ur->tcp_timeout)
+                                && parse_timeout(argv[optind++],
+                                                 &ur->tcp_fin_timeout)
+                                && parse_timeout(argv[optind++],
+                                                 &ur->udp_timeout);
+                        if (parse == 0)
+                                fail(2, "illegal timeout values");
+                } else
+                        fail(2, "-s option requires 3 timeout values");
+		break;
 	case 'h':
                 usage_exit(argv[0], 0);
-		break;
         default:
                 usage_exit(argv[0], -1);
         }
@@ -770,7 +816,7 @@ int parse_options(int argc, char **argv, int reading_stdin,
         }
 
         if (optind < argc)
-                fail(2, "unknown arguments found in command line");
+                fail(2, "unexpected arguments");
         return 0;
 }
 
@@ -1052,29 +1098,32 @@ void usage_exit(const char *program, const int exit_status) {
                 "       %s -[a|e] -[t|u|f] service-address -[r|R] server-address [-g|-i|-m] [-w weight]\n"
                 "       %s -d -[t|u|f] service-address -[r|R] server-address\n"
                 "       %s -[L|l] [-n]\n"
+                "       %s -s tcp tcpfin udp\n"
                 "       %s -h\n\n",
                 NVERSION(IP_VS_VERSION_CODE),                     
 #ifdef HAVE_POPT
                 program, program,
 #endif
-                program, program, program, program, program, program, program);
+                program, program, program,
+                program, program, program, program, program);
         
         fprintf(stream,
                 "Commands:\n"
                 "Either long or short options are allowed.\n"
                 "  --add-service     -A        add virtual service with options\n"
                 "  --edit-service    -E        edit virtual service with options\n"
-                "  --delete-service  -D	      delete virtual service\n"
+                "  --delete-service  -D        delete virtual service\n"
                 "  --clear           -C        clear the whole table\n"
 #ifdef HAVE_POPT
-                "  --restore         -R	      restore rules from stdin\n"
-                "  --save            -S	      save rules to stdout\n"
+                "  --restore         -R        restore rules from stdin\n"
+                "  --save            -S        save rules to stdout\n"
 #endif
                 "  --add-server      -a        add real server with options\n"
                 "  --edit-server     -e        edit real server with options\n"
                 "  --delete-server   -d        delete real server\n"
                 "  --list            -L|-l     list the table\n"
-                "  --help            -h	      display this help message\n\n"
+                "  --set|-s tcp tcpfin udp     set connection timeout values\n"
+                "  --help            -h        display this help message\n\n"
                 );
         
         fprintf(stream, 
